@@ -11,7 +11,7 @@ import sys
 import tempfile
 import warnings
 import webbrowser
-from typing import Dict, List, Tuple, Union
+from typing import Dict, List, Tuple, Union  # , Any
 
 import matplotlib.pyplot as plt
 from matplotlib.patches import Polygon, Circle, Arc
@@ -30,6 +30,8 @@ from dessia_common.typings import JsonSerializable
 import plot_data.colors
 from plot_data import templates
 
+# CURVES_DATATYPE = Union(List[float], List[str], List[List[float]], List[Dict[str, Any]])
+
 
 def delete_none_from_dict(dict1):
     """ Delete input dictionary's keys where value is None. """
@@ -46,7 +48,7 @@ def delete_none_from_dict(dict1):
 class PlotDataObject(DessiaObject):
     """ Abstract interface for DessiaObject implementation in module. """
 
-    _template_name = "empty_template"
+    _plot_commands = "EMPTY_TEMPLATE"
 
     def __init__(self, type_: str, name: str = '', **kwargs):
         self.type_ = type_
@@ -58,11 +60,6 @@ class PlotDataObject(DessiaObject):
             kwargs.pop('use_pointers')
         dict_ = DessiaObject.to_dict(self, use_pointers=False, **kwargs)
         del dict_['object_class']
-        package_name = self.__module__.split('.', maxsplit=1)[0]
-        if package_name in sys.modules:
-            package = sys.modules[package_name]
-            if hasattr(package, '__version__'):
-                dict_['package_version'] = package.__version__
 
         new_dict_ = delete_none_from_dict(dict_)
         return new_dict_
@@ -96,7 +93,8 @@ class Figure(PlotDataObject):
     @property
     def template(self):
         """ Get html template of current Figure object. """
-        return getattr(templates, self._template_name)
+        return templates.get_html_string(command_name=self._plot_commands)
+        # return getattr(templates, self._plot_commands)
 
     def _export_formats(self) -> List[ExportFormat]:
         """ Return a list of objects describing how to call generic exports (.json, .xlsx). """
@@ -368,6 +366,32 @@ DEFAULT_TEXTSTYLE = TextStyle(text_color=plot_data.colors.BLACK)
 DEFAULT_SURFACESTYLE = SurfaceStyle(color_fill=plot_data.colors.WHITE, opacity=1.)  # Not sure about opacity=1 in TS
 
 
+class Attribute(PlotDataObject):
+    """
+    Represents an attribute.
+
+    :param type_: The attribute's type (in that case, values are either 'float', 'color' or 'string')
+    :param name: The attribute's name
+    """
+
+    def __init__(self, type_: str, name: str):
+        PlotDataObject.__init__(self, type_=type_, name=name)
+
+
+class PointFamily(PlotDataObject):
+    """
+    A class that defines a point family. This class can be used in MultiplePlots to create families of points.
+
+    :param point_color: a color that is proper to this family (rgb255)
+    :param point_index: a list containing the point's index from MultiplePlots.elements
+    """
+
+    def __init__(self, point_color: str, point_index: List[int], name: str = ''):
+        self.color = point_color
+        self.point_index = point_index
+        PlotDataObject.__init__(self, type_=None, name=name)
+
+
 class Text(PlotDataObject):
     """
     A class for displaying texts on canvas. Text is a primitive and can be instantiated by PrimitiveGroup.
@@ -635,16 +659,12 @@ class Axis(PlotDataObject):
     :type grid_on: bool
     """
 
-    def __init__(self, nb_points_x: int = 10, nb_points_y: int = 10, graduation_style: TextStyle = None,
+    def __init__(self, nb_points_x: int = None, nb_points_y: int = None, graduation_style: TextStyle = None,
                  axis_style: EdgeStyle = None, arrow_on: bool = False, grid_on: bool = True, name: str = ''):
         self.nb_points_x = nb_points_x
         self.nb_points_y = nb_points_y
         self.graduation_style = graduation_style
-        if graduation_style is None:
-            self.graduation_style = TextStyle(text_color=plot_data.colors.GREY)
         self.axis_style = axis_style
-        if axis_style is None:
-            self.axis_style = EdgeStyle(color_stroke=plot_data.colors.LIGHTGREY)
         self.arrow_on = arrow_on
         self.grid_on = grid_on
         PlotDataObject.__init__(self, type_='axis', name=name)
@@ -744,8 +764,7 @@ class Graph2D(Figure):
     :type x_variable: str
     :param y_variable: variable that you want to display on y axis
     :type y_variable: str
-    :param axis: an object containing all information needed for \
-    drawing axis
+    :param axis: an object containing all information needed for drawing axis
     :type axis: Axis
     :param log_scale_x: True or False
     :type log_scale_x: bool
@@ -753,7 +772,7 @@ class Graph2D(Figure):
     :type log_scale_y: bool
     """
 
-    _template_name = "scatter_template"
+    _plot_commands = "GRAPH_COMMANDS"
 
     def __init__(self, graphs: List[Dataset], x_variable: str, y_variable: str, axis: Axis = None,
                  log_scale_x: bool = None, log_scale_y: bool = None, width: int = 750, height: int = 400,
@@ -783,6 +802,109 @@ class Graph2D(Figure):
         ax.set_xlabel(xname)
         ax.set_ylabel(yname)
         return ax
+# TODO: commented code here is supposed to be used soon
+    # def graphs_to_curves(self):
+    #     curves = []
+    #     for graph in self.graphs:
+    #         x_coords = []
+    #         y_coords = []
+    #         for sample in graph.elements:
+    #             x_coords.append(sample[self.attribute_names[0]])
+    #             y_coords.append(sample[self.attribute_names[1]])
+    #         line_width = graph.edge_style.line_width
+    #         color = graph.edge_style.color_stroke
+    #         dash_line = graph.edge_style.dashline
+    #         marker = graph.point_style.shape
+    #         name = graph.name
+    #         curves.append(Curve(x_coords, y_coords, name, line_width=line_width, color=color, dash_line=dash_line,
+    #                             marker=marker))
+    #     return curves
+
+    # def to_plot(self):
+    #     return
+
+
+# class Curve(PlotDataObject):
+
+#     _KWARGS = ['line_width', 'color', 'dash_line', 'marker']
+
+#     def __init__(self, x_coords: Union(List[str], List[float]), y_coords: Union(List[str], List[float]) = None,
+#                  name: str = '', **kwargs):
+#         self.x_coords, self.y_coords = self.buildCoords(x_coords, y_coords)
+#         self.line_width = None
+#         self.color = None
+#         self.dash_line = None
+#         self.marker = None
+#         self.setStyle(kwargs)
+
+#     @staticmethod
+#     def buildCoords(x_coords: Union(List[str], List[float]), y_coords: Union(List[str], List[float])):
+#         if y_coords is None:
+#             return list(range(len(y_coords))), x_coords
+#         if len(x_coords) == len(y_coords):
+#             return x_coords, y_coords
+#         raise ValueError("x_coords and y_coords must be the same length.")
+
+#     def setStyle(self, kwargs: Dict[str, Any]):
+#         for attribute in self._KWARGS:
+#             if attribute in kwargs:
+#                 setattr(self, attribute, kwargs[attribute])
+
+#     @classmethod
+#     def fromPlot(cls, x_values: CURVES_DATATYPE, y_values: CURVES_DATATYPE, x_variable: str, y_variable: str,
+#                  legend: List[str], **kwargs):
+#         if isinstance(x_values[0], float):
+#             if len(legend) != 1 and legend is not None:
+#                 raise ValueError("x_values and legend must be the same length.")
+#             if legend is None:
+#                 return cls(x_values, y_values, **kwargs)
+#             if len(legend) == 1:
+#                 return cls(x_values, y_values, legend[0], **kwargs)
+
+#         if isinstance(x_values[0], dict):
+#             if x_variable not in x_values[0]:
+#                 raise ValueError(f"{x_variable} not in keys of x_values.")
+
+#             x_coords = [];
+#             y_coords = [];
+#             for elements in x_values:
+#                 x_coords.append(elements[x_variable])
+#                 y_coords.append(elements[y_variable])
+#             return cls(x_coords, y_coords, legend[0], **kwargs)
+
+#         raise TypeError("x_values must be a list of float or dict.")
+
+
+# class Plot(Figure):
+
+#     _plot_commands = "GRAPH_COMMANDS"
+
+#     def __init__(self, x_values: CURVES_DATATYPE, y_values: CURVES_DATATYPE = None, x_variable: str = None,
+#                  y_variable: str = None, axis: Axis = None, legend: List[str] = None, width: int = 750,
+#                  height: int = 400, name: str = '', **kwargs):
+#         self.curves = self.buildCurves(x_values, y_values, x_variable, y_variable, legend, **kwargs)
+#         if axis is None:
+#             self.axis = Axis()
+#         else:
+#             self.axis = axis
+#         super().__init__(width=width, height=height, type_='graph2d', name=name)
+
+#     @staticmethod
+#     def buildCurves(x_values: CURVES_DATATYPE, y_values: CURVES_DATATYPE, x_variable: str, y_variable: str,
+#                     legend: List[str], **kwargs):
+#         if isinstance(x_values[0], (str, float, dict)):
+#             return [Curve.fromPlot(x_values=x_values, y_values=y_values, x_variable=x_variable, y_variable=y_variable,
+#                                    legend=legend, **kwargs)]
+
+#         if isinstance(x_values[0], list):
+#             curves = []
+#             for x_subvalues, y_subvalues, sub_legend in zip(x_values, y_values, legend):
+#                 curves.append(Curve.fromPlot(x_values=x_subvalues, y_values=y_subvalues, x_variable=x_variable,
+#                                              y_variable=y_variable, legend=sub_legend, **kwargs))
+#             return curves
+
+#         if isinstance(x_values[0], Curve):
+#             return x_values
 
 
 class Heatmap(DessiaObject):
@@ -824,12 +946,12 @@ class Scatter(Figure):
         If set to False, you'd still be able to enable it using the button.
     """
 
-    _template_name = "scatter_template"
+    _plot_commands = "SCATTER_COMMANDS"
 
-    def __init__(self, x_variable: str, y_variable: str, tooltip: Tooltip = None, point_style: PointStyle = None,
-                 elements: List[Sample] = None, axis: Axis = None, log_scale_x: bool = None, log_scale_y: bool = None,
-                 heatmap: Heatmap = None, heatmap_view: bool = None, width: int = 750, height: int = 400,
-                 name: str = ''):
+    def __init__(self, x_variable: str = None, y_variable: str = None, tooltip: Tooltip = None,
+                 point_style: PointStyle = None, elements: List[Sample] = None, points_sets: List[PointFamily] = None,
+                 axis: Axis = None, log_scale_x: bool = None, log_scale_y: bool = None, heatmap: Heatmap = None,
+                 heatmap_view: bool = None, width: int = 750, height: int = 400, name: str = ''):
         self.tooltip = tooltip
         self.attribute_names = [x_variable, y_variable]
         self.point_style = point_style
@@ -855,13 +977,14 @@ class Scatter(Figure):
         self.log_scale_y = log_scale_y
         self.heatmap = heatmap
         self.heatmap_view = heatmap_view
+        self.points_sets = points_sets
         super().__init__(width=width, height=height, type_='scatterplot', name=name)
 
 
 class ScatterMatrix(Figure):
     """ ScatterMatrix of a list of Samples. """
 
-    _template_name = "multiplot_template"
+    _plot_commands = "MULTIPLOT_COMMANDS"
 
     def __init__(self, elements: List[Sample] = None, axes: List[str] = None, point_style: PointStyle = None,
                  surface_style: SurfaceStyle = None, width: int = 750, height: int = 400, name: str = ""):
@@ -1056,7 +1179,7 @@ class PrimitiveGroup(Figure):
     Circle2D, Line2D, MultipleLabels, Wire, Point2D]]
     """
 
-    _template_name = "contour_template"
+    _plot_commands = "CONTOUR_COMMANDS"
 
     def __init__(self, primitives: List[Union[Contour2D, Arc2D, LineSegment2D, Circle2D,
                                               Line2D, MultipleLabels, Wire, Point2D]], width: int = 750,
@@ -1115,7 +1238,7 @@ class PrimitiveGroupsContainer(Figure):
     :type y_variable: str
     """
 
-    _template_name = "primitive_group_container_template"
+    _plot_commands = "PRIMITIVE_GROUP_CONTAINER_COMMANDS"
 
     def __init__(self, primitive_groups: List[PrimitiveGroup], sizes: List[Tuple[float, float]] = None,
                  coords: List[Tuple[float, float]] = None, associated_elements: List[int] = None,
@@ -1153,7 +1276,7 @@ class ParallelPlot(Figure):
         Color interpolation is enabled when clicking on an axis.
     """
 
-    _template_name = "parallelplot_template"
+    _plot_commands = "PARALLELPLOT_COMMANDS"
 
     def __init__(self, elements: List[Sample] = None, edge_style: EdgeStyle = None, disposition: str = None,
                  axes: List[str] = None, rgbs: List[Tuple[int, int, int]] = None, width: int = 750, height: int = 400,
@@ -1179,32 +1302,6 @@ class ParallelPlot(Figure):
         super().__init__(width=width, height=height, type_='parallelplot', name=name)
 
 
-class Attribute(PlotDataObject):
-    """
-    Represents an attribute.
-
-    :param type_: The attribute's type (in that case, values are either 'float', 'color' or 'string')
-    :param name: The attribute's name
-    """
-
-    def __init__(self, type_: str, name: str):
-        PlotDataObject.__init__(self, type_=type_, name=name)
-
-
-class PointFamily(PlotDataObject):
-    """
-    A class that defines a point family. This class can be used in MultiplePlots to create families of points.
-
-    :param point_color: a color that is proper to this family (rgb255)
-    :param point_index: a list containing the point's index from MultiplePlots.elements
-    """
-
-    def __init__(self, point_color: str, point_index: List[int], name: str = ''):
-        self.color = point_color
-        self.point_index = point_index
-        PlotDataObject.__init__(self, type_=None, name=name)
-
-
 class Histogram(Figure):
     """
     The Histogram object. This class can be instantiated in Multiplot.
@@ -1225,13 +1322,26 @@ class Histogram(Figure):
     :type surface_style: SurfaceStyle
     """
 
-    _template_name = "histogram_template"
+    _plot_commands = "HISTOGRAM_COMMANDS"
 
     def __init__(self, x_variable: str, elements=None, axis: Axis = None, graduation_nb: float = None,
                  edge_style: EdgeStyle = None, surface_style: SurfaceStyle = None, width: int = 750, height: int = 400,
                  name: str = ''):
+        if elements is None:
+            elements = []
+        sampled_elements = []
+        for element in elements:
+            # RetroCompat' < 0.11.0
+            if not isinstance(element, Sample) and isinstance(element, Dict):
+                reference_path = element.pop("reference_path", "#")
+                element_name = element.pop("name", "")
+                sampled_elements.append(Sample(values=element, reference_path=reference_path, name=element_name))
+            elif isinstance(element, Sample):
+                sampled_elements.append(element)
+            else:
+                raise ValueError(f"Element of type '{type(element)}' cannot be used as a MultiPlot data element.")
         self.x_variable = x_variable
-        self.elements = elements
+        self.elements = sampled_elements
         self.axis = axis
         self.graduation_nb = graduation_nb
         self.edge_style = edge_style
@@ -1251,7 +1361,7 @@ class MultiplePlots(Figure):
     :param initial_view_on: True for enabling initial layout, False  otherwise
     """
 
-    _template_name = "multiplot_template"
+    _plot_commands = "MULTIPLOT_COMMANDS"
 
     def __init__(self, plots: List[PlotDataObject], sizes: List[Window] = None, elements: List[Sample] = None,
                  coords: List[Tuple[float, float]] = None, point_families: List[PointFamily] = None,
